/***************************************************************************
      nnwview.c   -  draw a picture of the network
      version .03 -  demo for GTK, C
      -------------------
     begin                : Thu Apr 13 2000
     copyright            : (C) 2001 by Ed van der Meulen
     email                : ameulen@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************

 --------------------- print the NN ---------------------------------------*/
/* Nnwview is one of the viewing functions, the only in Demo 1. The        * 
 * ordering of cells and their relations in the NN are geographic. If we   *
 * number them the first as one etcetera and put only ther posnrs on a     *
 * 2-d sheet. Each cell starts after her mother and a sister under her     *
 * earlier born sis. Then we get a number of lines starting with a sister  *
 * and ending with an answer. In Demo1 are as much lines as we have letters* 
 * in our alphabet. A line starts under her former sister and not at the   *
 * left. Nnwview puts a '-1' on the places before a line starts. The lines *
 * are branches in the NN. In order to obtain one line of answers we insert*
 * a number of '0's at the rear of a branch so we get the answers in line. *
 * Then we turn the sheet 90 degrees clockwise and print the cells from    *
 * Alma to the answers and from right to left. Then Alma stands at the top.*
 * During print from above we have levels of cells. The horizontal space we*
 * divide by the number of cells on the level. If there is little room we  *
 * don't print the position numbers. Only after sleeping, high cells know  *
 * something of the answer below. To show we use GTK (the GimpTool Kit).   */

#include <gdk/gdk.h>
#include "nnw.h"
#include "nnwview.h"
#include <gtk/gtk.h>

/* Backing pixmap for drawing area */
static GdkPixmap *pixmap = NULL; // the canvas for the view

GdkColormap *cmap;               // our colormap for the pixmap
GdkColor blue;                   // our blue color
GdkColor grey;                   // same for grey
GdkGC *blue_gc;                  // blue gc
GdkGC *grey_gc;                  // grey gc

GdkFont *fixed;                  // our fixed font


static int yend = 0;             // greatest yc yet
static int mx = 0;               // greatest xc yet
static int ct = 0;		 // number of cell in the net yet
static int tabl[100][100];       // translate tabrl from coord x,y to celnr nc
static char well[256];		 // text in header of input written in the head of the view

static gint draw_nn (GtkWidget * widget)
{                                // put on the screen
    int scrw;    // width of the screen
    int scrh;    // height of the screen
    struct {int x; int y;} celpos[memmx];
    				 // celpos translates celnr to coord x,y
    				 // celpos is the inverse of tabl
    int orr[100];                // array sequence of the cells to print the NN
    int lt2;                     // local loop teller,        
    int nc;                      // local counter of cells
    int xc;                      // in landscape format horizontal x-axis coord 
    int yc;                      // vertical y-axis coordinate
    int lin = 0;                 // line number, special yc
    int spx, spy;                // x,y spaces in the text in the view 
    unsigned mxmax;              // max x in the GTK view
    short krap;                  // boolean too wide for all info for this screen
    int ryc;                     // real y counter, physical y pos on screen
    int rxc;                     // real x, like ryc
    int txt_wdth;                // text width
    int cell_height;             // height of a cell
    int half_cell_height;        // half height of a cell
    int tans = 0;                // temporary ans
    int tmar = 0;                // temporary mar
    int hor,ver,lt;		 // local used
    char txt[256];

    /* create graphic context for the line color, blue */
    blue_gc = gdk_gc_new (widget->window);
    gdk_gc_set_foreground (blue_gc, &blue);
    grey_gc = gdk_gc_new (widget->window);
    gdk_gc_set_foreground (grey_gc,&grey);
    //----------------------------------------------------------------
    ver = mx + 1;                       // turning 90 degrees x-as becomes vertical
    hor = yend + 1;			// ver and hor are here the sizes of the view on the schreen
    scrh = widget->allocation.height;   // inquiring screen heigth and width
    scrw = widget->allocation.width;

    cell_height = line_height << 1;
    spy = (scrh - top_spacing - bottom_spacing) / ver;  // not an NNW way to use, but making a view 
    							// is NOT natural
    							// horizontal distance between cell info's
    if ( strstr(well, "doozed")==0 ||
	    spy == 0 ||
	    (hor * cell_width * char_width) > scrw )
    {
	krap = 1;
	cell_height >>= 1;              // it takes only one line for a cell if krap
    }
    else
	krap = 0;                       // krap denotes a screen to smal for all info   
    ryc = top_spacing - spy;            // left up y pos starts spy above topside
    half_cell_height = cell_height >> 1;

    //---------------------------------- the view loop ----------------------------
    for (xc = 0; xc <= mx; xc++)        // work here per column, certain xc, from top
    {                                   // xc is a horizonntal level from top to bottom
	yc = yend;                      // yc is set on the left cell of this level
	lt = 0;				// so we mirror along the vertical axis
	orr[0] = 0;			// orr holds the connection between new nc in celpos[nc]
	do				// and the original nc
	{                               // fill the set orr of posnr of level xc
	    nc = tabl[xc][yc];          // nc is such a pos number od a cell
	    if (nc < 0)
	    {
		yc--;                   // skip the fore runner cells -1 completely
		continue;         
	    }
	    lt++;
	    orr[lt] = nc;               // orr is numbering the nc from yend thru 0 
	    yc--;                       // for fixed level xc
	}   while (yc >= 0);            // at the and yc=0, lt=width of level

	spx = scrw / (lt + 1);          // can it be done without dividing?
	txt[0] = '\0';

	// initiate ryc and rxc 
	ryc += spy;                     // increase ryc, one line more down...
	rxc = scrw;                     // first cell is at most right pos

	for (lt2 = 1; lt2 <= lt; lt2++) // lt is the width of the level
	{                               // so copy the one horizontal level
	    celpos[orr[lt2]].y = ryc;           // y pos stays equal on one line
	    celpos[orr[lt2]].x = (rxc -= spx);  // 1st cell is right, x decreases
	    // with spx, the x space between
	    // centers of the cells

	    if ((orr[lt2] == 0) && (xc > 0))    // give fillers for shifted answers
		continue;                 // a blank position, resulting in a hole
	    lin = cel[orr[lt2]].ans;      // but never our Alma
	    if (cel[orr[lt2]].mar != 32767)	// in future platform independent
	    {                             // ans,mar .. used as .. lowerbound,upperbound
		tans = (lin - cel[orr[lt2]].mar) >> 1; // just temporary
		tmar = (lin + cel[orr[lt2]].mar) >> 1;
	    }                             // in future always low and up bound ??

	    if (!krap || cel[orr[lt2]].dot == 0)
	    {
		txt[0] = tans;            // txt is the string low-up bounds
		txt[1] = '-';             // type casting to string low-up bounds
		txt[2] = tmar;
		txt[3] = '\0';            // in H high the NN learns what he can do
		mxmax = gdk_string_width (fixed, txt);
		gdk_draw_string (widget->window,
			fixed,      	  // draw the string with answers---------
			widget->style->white_gc,
			celpos[orr[lt2]].x - (mxmax >> 1) + 1,// text centered
			celpos[orr[lt2]].y + half_cell_height, txt);

		if (!krap)
		{   
		    sprintf (txt, "%i", orr[lt2]); // txt contains now the pos nr
		    mxmax = (mxmax < (txt_wdth = 
				gdk_string_width (fixed, txt)) ? txt_wdth : mxmax);
		    gdk_draw_string (widget->window,
			    fixed,        // draw the position number-------------
			    widget->style->white_gc,
			    celpos[orr[lt2]].x - (txt_wdth >> 1),// centered
			    celpos[orr[lt2]].y, 
			    txt);
		}   
	    }
	    else
	    {                             // krap
		if (orr[lt2]==0)
		{
		    sprintf (txt, "%i", orr[lt2]);
		    mxmax=gdk_string_width(fixed,txt);
		    gdk_draw_string (widget->window,
			    fixed,        // draw the pos nr of Alma-------------
			    widget->style->white_gc,
			    celpos[orr[lt2]].x - (mxmax >> 1), // center
			    celpos[orr[lt2]].y,
			    txt);
		}
		else
		{
		    strcpy(txt,"-");
		    mxmax=gdk_string_width(fixed,txt);
		    gdk_draw_string (widget->window,
			    fixed,        // draw if krap only a short line---------
			    grey_gc,
			    celpos[orr[lt2]].x,
			    celpos[orr[lt2]].y + half_cell_height, txt);
		}
	    }
	    if (orr[lt2] > 0)
	    {
		gdk_draw_line (widget->window,
			blue_gc,          // draw line from mother to this daughter---
			celpos[cel[orr[lt2]].mot].x,
			celpos[cel[orr[lt2]].mot].y +
			half_cell_height + under_spacing,
			celpos[orr[lt2]].x,
			celpos[orr[lt2]].y - half_cell_height - upper_spacing);
	    }

	    if (cel[orr[lt2]].dot == 0)         // for answer cells at the bottom
		gdk_draw_line (widget->window,  // draw a longer horizontal line--
			widget->style->white_gc,
			celpos[orr[lt2]].x - (mxmax >> 1) - 1,
			celpos[orr[lt2]].y + half_cell_height + under_margin,
			celpos[orr[lt2]].x + ((mxmax + 1) >> 1) + 2,
			celpos[orr[lt2]].y + half_cell_height + under_margin);

	    gdk_draw_string (widget->window,
		    fixed, widget->style->white_gc, 0, line_height,
		    well);         // draw headers-----------------------------

	    sprintf (txt, "Number of cells: %i", ct);
	    gdk_draw_string (widget->window,
		    fixed, widget->style->white_gc, 55,
		    (line_height << 1), txt);

	}
	//---------------------The end-----------------------------------
    }
    return TRUE;
}


/* Create a new backing pixmap of the appropriate size */
static gint configure_event (GtkWidget * widget, GdkEventConfigure * event)
{
    if (pixmap)   gdk_pixmap_unref (pixmap);

    pixmap = gdk_pixmap_new (widget->window,
	    widget->allocation.width,
	    widget->allocation.height, -1);

    gdk_draw_rectangle (pixmap,
	    widget->style->black_gc,
	    TRUE,
	    0, 0,
	    widget->allocation.width, widget->allocation.height);
    draw_nn (widget);
    return TRUE;
}

/* Redraw the screen from the backing pixmap */
static gint expose_event (GtkWidget * widget, GdkEventExpose * event)
{
    gdk_draw_pixmap (widget->window,
	    widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
	    pixmap,
	    event->area.x, event->area.y,
	    event->area.x, event->area.y,
	    event->area.width, event->area.height);
    draw_nn (widget);
    return FALSE;
}

void quit (GtkWidget *widget)
{
    gtk_main_quit();
}

int nnwview(char *fin)
{
    char name[] = "nnwview";
    char version[] = ".03";
    char wtitle[256];                // title of the GTK - window
    int lenx[100];                   // length in number of xc of a branch yc
    int stanc[100];                  // start nc of a branch yc
    int endnc[100];                  // last nc number of a branch yc
    int vlo[100];                    // front run points before a branch starts
    int celnr[200];                  // array but not used
    char *q;
    int hor,ver;
    int xc,yc,nc,lt,lin;
    FILE *fi;
    long dat1,dat2,dat3;

    GtkWidget *window;
    GtkWidget *drawing_area;
    GtkWidget *vbox;

    GtkWidget *button;               // quit button

    ct = 0;
    memset (endnc, 0, 100 * sizeof(int));
    memset (tabl, 0, 100*100 * sizeof(int));


    if ((fi = fopen (fin, "r")) == NULL) // consult and change the cell
    {
	printf("nnwview.c: error opening file %s\n",fin);
	return 1;
    }

    //----------------------------- read NN per cell (line) ---------------

    fscanf(fi, "%li %li %li %i %i", &dat1, &dat2, &dat3, &hor, &ver);//admin line
    fgets (well, 255, fi);
    if ((q = strchr (well, '\n')) != NULL) // well gets its text
	*q = '\0';
    while (!feof (fi))			   // read per cell
    {
	fscanf (fi, "%i %i %i %i %i %i %i %i %i %i", &cel[ct].ad, &cel[ct].me,
		&cel[ct].wei, &cel[ct].bi, &cel[ct].mot, &cel[ct].sis,
		&cel[ct].dot, &cel[ct].nan, &cel[ct].ans, &cel[ct].mar);
	ct++;
    }
    //------ reading of NN done -------------------------------------

    ct -= 2;
    fclose (fi);

    celnr[0] = 0;

    //------------- put celnrs from NN in tabl ---------------
    xc = 0;                       // orig x coord
    yc = 0;                       // orig y coord
    nc = 0;                       // a cellnumber
    mx = 0;                       // greatest x

    vlo[0] = 0;                   // front runners for branch starts per yc
    tabl[0][0] = 0;               // table points from coord x=0,y=0 to celnr nc=0
    stanc[0] = 0;                 // branch starts with this nc
    do
    {
	do
	{
	    nc++;
	    xc++;                 // put cells as branches
	    tabl[xc][yc] = nc;    // tabl points from coord xc,yc to celnr nc  
	}
	while (cel[nc].dot != 0); // branch ends with aswer dot = 0
	endnc[yc + 1] = nc;       // branches count from 1, and ends at cell nc
	lenx[yc] = xc;            // length of branch yc
	if (xc > mx)
	    mx = xc;              // mx farest right reaching cell
	nc++;                     // nc to the next branch
	yc++;                     // this is the next branch
	stanc[yc] = nc;           // line yc starts with celnr nc 
	lt = cel[nc].sis;         // her sister on a former line
	if (lt != nc)             // search sister with lower celnr
	    while (lt > nc)
		lt = cel[lt].sis; // my first sister
	lin = yc;
	while (lt < endnc[lin]) {
	    lin--;                // search the line of my first sister
	}
	vlo[yc] = vlo[lin] + lt - stanc[lin];   // fore runners
	for (xc = 0; xc <= vlo[yc]; xc++)
	    tabl[xc][yc] = -1;    // fore runners are -1
	xc -= 2;
	nc--;
    }
    while (nc < ct);              // till all cells in table
    yend = yc - 1;

    for (yc = 0; yc <= yend; yc++)
    {                             // shift answer cells to rightest column
	xc = lenx[yc];
	if (xc < mx)
	{
	    tabl[mx][yc] = tabl[xc][yc];
	    tabl[xc][yc] = 0;
	}
    }

    /* new blue color */
    cmap = gdk_colormap_get_system ();
    blue.red   = blue_red * (65535 / 255); // laziness ??
    blue.green = blue_green * (65535 / 255);
    blue.blue  = blue_blue * (65535 / 255);
    grey.red   = grey_red * (65535/255);
    grey.green = grey_green * (65535/255);
    grey.blue  = grey_blue * (65535/255);
    if (!gdk_colormap_alloc_color (cmap, &blue, FALSE, TRUE)
	    || (blue.pixel == 0))
    {
	g_error ("couldn't allocate color");
    }
    if (!gdk_colormap_alloc_color (cmap, &grey, FALSE, TRUE)
	    || (grey.pixel == 0))
    {
	g_error ("couldn't allocate color");
    }



    /* Load fixed font */
    fixed = gdk_font_load ("fixed");

    /* Create window and title */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    strcpy (wtitle, name);
    strcat (wtitle, " ");
    strcat (wtitle, version);
    gtk_widget_set_name (window, wtitle);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
	    GTK_SIGNAL_FUNC (quit), GTK_OBJECT (window));

    /* Create the initial drawing area */
    drawing_area = gtk_drawing_area_new ();
    gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), scw, sch);
    gtk_box_pack_start (GTK_BOX (vbox), drawing_area, TRUE, TRUE, 0);

    gtk_widget_show (drawing_area);

    /* Signals used to handle backing pixmap */

    gtk_signal_connect (GTK_OBJECT (drawing_area), "expose_event",
	    (GtkSignalFunc) expose_event, NULL);
    gtk_signal_connect (GTK_OBJECT (drawing_area), "configure_event",
	    (GtkSignalFunc) configure_event, NULL);

    /* .. And a quit button */
    button = gtk_button_new_with_label ("Quit");
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);

    gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
	    GTK_SIGNAL_FUNC (gtk_widget_destroy),
	    GTK_OBJECT (window));
    gtk_widget_show (button);


    gtk_widget_show (window);

    //----------------------------Start body----------------------------------------
    gtk_main ();

    return EXIT_SUCCESS;          // thanks, GTK team
}
