/***************************************************************************
      patterns.c  -  Convert text to snnslet bit-patterns
      project   -  Nnw
      version   -   .03
      -------------------
     begin                : July 18th 2001
     copyright            : (C) 2001 by Charles Baker
     email                : telegramsam_la@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* This is the new and improved patterns.c file that converts plain-text to SNNSLET bit-patterns
 * that are used by Nnw. The program takes 2 arguments to the command as follows:
 * patterns [inputfile] [outputfile]
 * Currently supported characters are: A-Z, a-z, 0-9 and the following punctuation:
 * <space> . : ; " ! ' , ( ) / \ ? * # - + = _
 */
 
#include <stdio.h>
#include <string.h>

#define MAX_INT 35
#define MAX_CHAR 69
#define ARGS 3

void fill_pattern(int chr, char *pat);
int snnslet_convert(char *in, char *out);

int main(int argc, char *argv[]) {
    char pattern[MAX_CHAR], *pat, *in, *out;
    pat = pattern;

    if(argc != ARGS) {
	printf("Usage: patterns [input file] [output file]\n");
	exit(1);
    }
    in = argv[1];
    out = argv[2];
    if(snnslet_convert(in, out)) {
	printf("Error in conversion process...bailing now\n");
	exit(1);
    }
    printf("Conversion Complete!\n");
    return 0;
}

// this will take the input file in human format and transfer to snnslet output file
int snnslet_convert(char *in, char *out) {
    FILE *infile, *outfile;
    long pos = 1;
    char chin, stout[MAX_CHAR];

    if((infile = fopen(in, "r")) == NULL) {
	printf("Error opening input file: %s\n", in);
	exit(1);
    }
    if((outfile = fopen(out, "w")) == NULL) {
	printf("Error opening output file: %s\n", out);
	exit(1);
    }
    while((chin = fgetc(infile)) != EOF) {
	fseek(infile, pos, SEEK_SET);
	fill_pattern(chin, stout);
	fprintf(outfile, "%s", stout);
	pos++;
    }
    printf("%ld Characters read from input file\n", pos);
    fclose(infile);
    fclose(outfile);
    return 0;
}

// this function checks the input character and fills the pattern according to the character.
// a little bit of tedium here saves us lots of time in the future ;)
void fill_pattern(int chr, char *pat) {

    switch(chr) {
	// XXX: make these patterns available in some 'snnslet' file too, currently
	//      only numbers and capital letters are there, these patterns shouldn't
	//      be hardcoded anyway

	// Lowercase:
	case 'a':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 0\n1 0 0 0 1\n0 0 0 0 1\n0 1 1 1 1\n1 0 0 0 1\n0 1 1 1 1\n97\n");
	    break;
	case 'b':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 0\n98\n");
	    break;
	case 'c':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 1\n0 1 1 1 0\n99\n");
	    break;
	case 'd':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 0 1\n0 0 0 0 1\n0 1 1 1 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 1\n100\n");
	    break;
	case 'e':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 0\n1 0 0 0 1\n1 1 1 1 1\n1 0 0 0 0\n1 0 0 0 1\n0 1 1 1 0\n101\n");
	    break;
	case 'f':
	    strcpy(pat, "0 0 0 0 0\n0 0 1 1 0\n0 1 0 0 0\n1 1 1 0 0\n0 1 0 0 0\n0 1 0 0 0\n0 1 0 0 0\n102\n");
	    break;
	case 'g':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 0\n1 0 0 0 1\n0 1 1 1 1\n0 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n103\n");
	    break;
	case 'h':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n104\n");
	    break;
	case 'i':
	    strcpy(pat, "0 0 0 0 0\n0 0 1 0 0\n0 0 0 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n105\n");
	    break;
	case 'j':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 1 0\n0 0 0 0 0\n0 0 0 1 0\n0 0 0 1 0\n1 0 0 1 0\n0 1 1 0 0\n106\n");
	    break;
	case 'k':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 0\n1 0 0 1 0\n1 0 1 0 0\n1 1 0 0 0\n1 0 1 0 0\n1 0 0 1 0\n107\n");
	    break;
	case 'l':
	    strcpy(pat, "0 0 0 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n108\n");
	    break;
	case 'm':
	    strcpy(pat, "0 0 0 0 0\n0 1 0 1 0\n1 0 1 0 1\n1 0 1 0 1\n1 0 1 0 1\n1 0 0 0 1\n1 0 0 0 1\n109\n");
	    break;
	case 'n':
	    strcpy(pat, "0 0 0 0 0\n1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n110\n");
	    break;
	case 'o':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n111\n");
	    break;
	case 'p':
	    strcpy(pat, "0 0 0 0 0\n1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 0\n1 0 0 0 0\n1 0 0 0 0\n112\n");
	    break;
	case 'q':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 1\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 1\n0 0 0 0 1\n0 0 0 0 1\n113\n");
	    break;
	case 'r':
	    strcpy(pat, "0 0 0 0 0\n1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n114\n");
	    break;
	case 's':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 0\n1 0 0 0 1\n1 1 1 1 0\n0 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n115\n");
	    break;
	case 't':
	    strcpy(pat, "0 0 0 0 0\n0 0 1 0 0\n1 1 1 1 1\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n116\n");
	    break;
	case 'u':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n117\n");
	    break;
	case 'v':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 0 1 0\n0 0 1 0 0\n118\n");
	    break;
	case 'w':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 1 0 1\n1 0 1 0 1\n0 1 0 1 0\n119\n");
	    break;
	case 'x':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 1\n0 1 0 1 0\n0 0 1 0 0\n0 0 1 0 0\n0 1 0 1 0\n1 0 0 0 1\n120\n");
	    break;
	case 'y':
	    strcpy(pat, "0 0 0 0 0\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 1\n0 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n121\n");
	    break;
	case 'z':
	    strcpy(pat, "0 0 0 0 0\n1 1 1 1 1\n0 0 0 1 0\n0 0 1 0 0\n0 1 0 0 0\n1 0 0 0 0\n1 1 1 1 1\n122\n");
	    break;
	    // Uppercase:
	case 'A':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n65\n");
	    break;
	case 'B':
	    strcpy(pat, "1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 0\n66\n");
	    break;
	case 'C':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 1\n0 1 1 1 0\n67\n");
	    break;
	case 'D':
	    strcpy(pat, "1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 0\n68\n");
	    break;
	case 'E':
	    strcpy(pat, "1 1 1 1 1\n1 0 0 0 0\n1 0 0 0 0\n1 1 1 1 0\n1 0 0 0 0\n1 0 0 0 0\n1 1 1 1 1\n69\n");
	    break;
	case 'F':
	    strcpy(pat, "1 1 1 1 1\n1 0 0 0 0\n1 0 0 0 0\n1 1 1 1 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n70\n");
	    break;
	case 'G':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 0\n1 0 1 1 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n71\n");
	    break;
	case 'H':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n72\n");
	    break;
	case 'I':
	    strcpy(pat, "0 1 1 1 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 1 1 1 0\n73\n");
	    break;
	case 'J':
	    strcpy(pat, "1 1 1 1 1\n0 0 0 0 1\n0 0 0 0 1\n0 0 0 0 1\n0 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n74\n");
	    break;
	case 'K':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 1 0\n1 0 1 0 0\n1 1 0 0 0\n1 0 1 0 0\n1 0 0 1 0\n1 0 0 0 1\n75\n");
	    break;
	case 'L':
	    strcpy(pat, "1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n1 1 1 1 1\n76\n");
	    break;
	case 'M':
	    strcpy(pat, "1 0 0 0 1\n1 1 0 1 1\n1 0 1 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n77\n");
	    break;
	case 'N':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 0 1\n1 1 0 0 1\n1 0 1 0 1\n1 0 0 1 1\n1 0 0 0 1\n1 0 0 0 1\n78\n");
	    break;
	case 'O':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n79\n");
	    break;
	case 'P':
	    strcpy(pat, "1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 0\n1 0 0 0 0\n1 0 0 0 0\n1 0 0 0 0\n80\n");
	    break;
	case 'Q':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 1 0 1\n1 0 0 1 1\n0 1 1 1 1\n81\n");
	    break;
	case 'R':
	    strcpy(pat, "1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 0\n1 0 1 0 0\n1 0 0 1 0\n1 0 0 0 1\n82\n");
	    break;
	case 'S':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 0\n0 1 1 1 0\n0 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n83\n");
	    break;
	case 'T':
	    strcpy(pat, "1 1 1 1 1\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n84\n");
	    break;
	case 'U':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n85\n");
	    break;
	case 'V':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 0 1 0\n0 0 1 0 0\n86\n");
	    break;
	case 'W':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 1 0 1\n1 0 1 0 1\n0 1 0 1 0\n87\n");
	    break;
	case 'X':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 0 1\n0 1 0 1 0\n0 0 1 0 0\n0 1 0 1 0\n1 0 0 0 1\n1 0 0 0 1\n88\n");
	    break;
	case 'Y':
	    strcpy(pat, "1 0 0 0 1\n1 0 0 0 1\n0 1 0 1 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n89\n");
	    break;
	case 'Z':
	    strcpy(pat, "1 1 1 1 1\n0 0 0 0 1\n0 0 0 1 0\n0 0 1 0 0\n0 1 0 0 0\n1 0 0 0 0\n1 1 1 1 1\n90\n");
	    break;
	    // Numerals:
	case '0':
	    strcpy(pat, "0 0 0 0 0\n0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n48\n");
	    break;
	case '1':
	    strcpy(pat, "0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n49\n");
	    break;
	case '2':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n0 0 0 1 0\n0 0 1 0 0\n0 1 0 0 0\n1 1 1 1 1\n50\n");
	    break;
	case '3':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n0 0 0 1 0\n0 1 1 0 0\n0 0 0 1 0\n1 0 0 0 1\n0 1 1 1 0\n51\n");
	    break;
	case '4':
	    strcpy(pat, "0 0 0 0 1\n1 0 0 0 1\n1 0 0 0 1\n1 1 1 1 1\n0 0 0 0 1\n0 0 0 0 1\n0 0 0 0 1\n52\n");
	    break;
	case '5':
	    strcpy(pat, "1 1 1 1 1\n1 0 0 0 0\n1 0 0 0 0\n1 1 1 1 0\n0 0 0 0 1\n0 0 0 0 1\n1 1 1 1 0\n53\n");
	    break;
	case '6':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 0\n1 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n54\n");
	    break;
	case '7':
	    strcpy(pat, "1 1 1 1 1\n0 0 0 1 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n55\n");
	    break;
	case '8':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 0\n56\n");
	    break;
	case '9':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n1 0 0 0 1\n0 1 1 1 1\n0 0 0 0 1\n0 0 0 0 1\n0 0 0 0 1\n57\n");
	    break;
	    // Punctuation:
	case ' ':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n32\n");
	    break;
	case '.':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 1 0 0\n0 1 1 1 0\n0 0 1 0 0\n46\n");
	    break;
	case ':':
	    strcpy(pat, "0 0 0 0 0\n0 0 1 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 1 0 0\n0 0 0 0 0\n58\n");
	    break;
	case ';':
	    strcpy(pat, "0 0 0 0 \n0 0 1 0 0\n0 0 0 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 1 0 0 0\n59\n");
	    break;
	case '"':
	    strcpy(pat, "0 1 0 1 0\n0 1 0 1 0\n0 1 0 1 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n34\n");
	    break;
	case '!':
	    strcpy(pat, "0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 0 0 0\n0 0 1 0 0\n33\n");
	    break;
	case '\'':
	    strcpy(pat, "0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n39\n");
	    break;
	case ',':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 1 0 0 0\n44\n");
	    break;
	case '(':
	    strcpy(pat, "0 0 0 0 1\n0 0 0 1 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 0 1 0\n0 0 0 0 1\n40\n");
	    break;
	case ')':
	    strcpy(pat, "1 0 0 0 0\n0 1 0 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 1 0 0\n0 1 0 0 0\n1 0 0 0 0\n41\n");
	    break;
	case '/':
	    strcpy(pat, "0 0 0 0 1\n0 0 0 1 0\n0 0 0 1 0\n0 0 1 0 0\n0 1 0 0 0\n0 1 0 0 0\n1 0 0 0 0\n47\n");
	    break;
	case '\\':
	    strcpy(pat, "1 0 0 0 0\n0 1 0 0 0\n0 1 0 0 0\n0 0 1 0 0\n0 0 0 1 0\n0 0 0 1 0\n0 0 0 0 1\n92\n");
	    break;
	case '?':
	    strcpy(pat, "0 1 1 1 0\n1 0 0 0 1\n0 0 0 1 0\n0 0 1 0 0\n0 0 1 0 0\n0 0 0 0 0\n0 0 1 0 0\n63\n");
	    break;
	case '*':
	    strcpy(pat, "0 0 0 0 0\n1 0 1 0 1\n0 1 1 1 0\n1 1 1 1 1\n0 1 1 1 0\n1 0 1 0 1\n0 0 0 0 0\n42\n");
	    break;
	case '#':
	    strcpy(pat, "0 0 0 0 0\n0 1 0 1 0\n1 1 1 1 1\n0 1 0 1 0\n1 1 1 1 1\n0 1 0 1 0\n0 0 0 0 0\n35\n");
	    break;
	case '-':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n1 1 1 1 1\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n45\n");
	    break;
	case '+':
	    strcpy(pat, "0 0 0 0 0\n0 0 1 0 0\n0 0 1 0 0\n1 1 1 1 1\n0 0 1 0 0\n0 0 1 0 0\n0 0 0 0 0\n43\n");
	    break;
	case '=':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 0 0\n1 1 1 1 1\n0 0 0 0 0\n1 1 1 1 1\n0 0 0 0 0\n0 0 0 0 0\n61\n");
	    break;
	case '_':
	    strcpy(pat, "0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n0 0 0 0 0\n1 1 1 1 1\n95\n");
	    break;

    }
}
